package logic

import (
	"fusenapi/model/gmodel"
	"fusenapi/utils/basic"
	"fusenapi/utils/email"
	"net/http"
	"strings"
	"time"

	"context"

	"fusenapi/server/ldap-admin/internal/svc"
	"fusenapi/server/ldap-admin/internal/types"

	"github.com/zeromicro/go-zero/core/logx"
)

type UpdateLdapUserPwdLogic struct {
	logx.Logger
	ctx    context.Context
	svcCtx *svc.ServiceContext
}

func NewUpdateLdapUserPwdLogic(ctx context.Context, svcCtx *svc.ServiceContext) *UpdateLdapUserPwdLogic {
	return &UpdateLdapUserPwdLogic{
		Logger: logx.WithContext(ctx),
		ctx:    ctx,
		svcCtx: svcCtx,
	}
}

// 处理进入前逻辑w,r
// func (l *UpdateLdapUserPwdLogic) BeforeLogic(w http.ResponseWriter, r *http.Request) {
// }

func (l *UpdateLdapUserPwdLogic) UpdateLdapUserPwd(req *types.UpdateLdapUserPwdReq, r *http.Request) (resp *basic.Response) {

	if !l.svcCtx.Ldap.VerifyAuthority(r) {
		return resp.SetStatusWithMessage(basic.CodeUnAuth, "无权限，请联系管理员开通")
	}
	req.UserDN = strings.Trim(req.UserDN, " ")
	req.NewPassword = strings.Trim(req.NewPassword, " ")
	req.OldPassword = strings.Trim(req.OldPassword, " ")
	if req.NewPassword == "" || req.OldPassword == "" {
		return resp.SetStatusWithMessage(basic.CodeRequestParamsErr, "新/旧密码都不允许为空")
	}
	if len(req.UserDN) <= 3 || req.UserDN[:3] != "cn=" {
		return resp.SetStatusWithMessage(basic.CodeRequestParamsErr, "参数错误，无效的用户DN")
	}
	cnEmail := strings.Split(req.UserDN, ",")[0][3:]
	if !email.IsEmailValid(cnEmail) {
		return resp.SetStatusWithMessage(basic.CodeRequestParamsErr, "错误的用户cn")
	}
	//查询个人信息
	user, err := l.svcCtx.Ldap.GetLdapUserInfo(req.UserDN)
	if err != nil {
		logx.Error(err)
		return resp.SetStatusWithMessage(basic.CodeServiceErr, err.Error())
	}
	if user.Password != req.OldPassword {
		return resp.SetStatusWithMessage(basic.CodeServiceErr, "旧密码不对，请重新尝试")
	}
	err = l.svcCtx.Ldap.Update(req.UserDN, map[string][]string{
		"userPassword": {req.NewPassword},
	})
	if err != nil {
		logx.Error(err)
		return resp.SetStatusWithMessage(basic.CodeServiceErr, "修改密码失败")
	}
	now := time.Now().UTC()
	err = l.svcCtx.AllModels.LdapUser.Update(l.ctx, req.UserDN, &gmodel.LdapUser{
		Utime: &now,
	})
	if err != nil {
		logx.Error(err)
	}
	return resp.SetStatusWithMessage(basic.CodeOK, "修改密码成功")
}

// 处理逻辑后 w,r 如:重定向, resp 必须重新处理
// func (l *UpdateLdapUserPwdLogic) AfterLogic(w http.ResponseWriter, r *http.Request, resp *basic.Response) {
// // httpx.OkJsonCtx(r.Context(), w, resp)
// }
